
#include <cstdio>
#include <new>
#include <utility>

enum ErrorCode {
    OK,

    ERR_MEM,
    ERR_VALUE,
};

static thread_local ErrorCode error;

template <typename T>
struct Owner {
    T *ptr;
    Owner(T *p = nullptr) : ptr(p) {}
    ~Owner() {
        if (ptr) {
            delete ptr;
        }
    }

    template <typename... Args>
    void alloc(Args &&...args) {
        ptr = new (std::nothrow) T(std::forward<Args>(args)...);
    }
};

struct C {
    C() : i(0) {}
    int i;
};

struct D {
    D(int a, int b) : c(a), k(b) {
        if (k > 5){
            error = ERR_VALUE;
        }
    }
    int c;
    int k;
};

int main() {
    int retval = -1;

    Owner<C> c;
    c.alloc();
    if (! c.ptr) {
        printf("C alloc error\n");
        error = ERR_MEM;
        return 1;
        // no need to do anything here because ~Owner<C>() took care of cleanup
    }

    Owner<D> d;
    d.alloc(7, 9);
    if (! d.ptr) {
        printf("D alloc error\n");
        error = ERR_MEM;
        return 2;
        // no need to do anything here because ~Owner<C>() took care of cleanup
    }

    if (error != OK) {
        printf("D alloc err: %d\n", error);
        retval = error;
        return 3;
        // no need to do anything here because ~Owner<C>()
        // and ~Owner<D>() took care of cleanup
    }

    return 0;
    // no need to do anything here because ~Owner<C>()
    // and ~Owner<D>() took care of cleanup
}
